@@ -6,6 +6,6 @@
  *
  * Returns 0 if no diagnostic message has been found else 1.
  */
-int cfg_run_diagnostics();
+int cfg_run_diagnostics(void);
 
 #endif /* _HAPROXY_CFGDIAG_H */
