@@ -45,15 +45,19 @@
 #include <haproxy/uri_auth-t.h>
 #include <haproxy/http_ext-t.h>
 
-/* values for proxy->mode */
+/* values for proxy->mode, only one value per proxy.
+ *
+ * values are bitfield compatible so that functions may
+ * take a bitfield of compatible modes as parameter
+ */
 enum pr_mode {
-	PR_MODE_TCP = 0,
-	PR_MODE_HTTP,
-	PR_MODE_CLI,
-	PR_MODE_SYSLOG,
-	PR_MODE_PEERS,
-	PR_MODE_SPOP,
-	PR_MODES
+	PR_MODES       = 0x00,
+	PR_MODE_TCP    = 0x01,
+	PR_MODE_HTTP   = 0x02,
+	PR_MODE_CLI    = 0x04,
+	PR_MODE_SYSLOG = 0x08,
+	PR_MODE_PEERS  = 0x10,
+	PR_MODE_SPOP   = 0x20,
 } __attribute__((packed));
 
 enum PR_SRV_STATE_FILE {
@@ -110,11 +114,12 @@ enum PR_SRV_STATE_FILE {
 #define PR_O_HTTP_CLO   0x01000000      /* HTTP close mode (httpclose) */
 #define PR_O_HTTP_SCL   0x02000000      /* HTTP server close mode (http-server-close) */
 #define PR_O_HTTP_MODE  0x03000000      /* MASK to retrieve the HTTP mode */
-/* unused: 0x04000000 */
 
-#define PR_O_TCPCHK_SSL 0x08000000	/* at least one TCPCHECK connect rule requires SSL */
-#define PR_O_CONTSTATS	0x10000000	/* continuous counters */
-/* unused: 0x20000000 */
+#define PR_O_HTTP_DROP_REQ_TRLS 0x04000000 /* Drop the request trailers when forwarding to the server */
+#define PR_O_HTTP_DROP_RES_TRLS 0x08000000 /* Drop response trailers when forwarding to the client */
+
+#define PR_O_TCPCHK_SSL 0x10000000      /* at least one TCPCHECK connect rule requires SSL */
+#define PR_O_CONTSTATS  0x20000000      /* continuous counters */
 #define PR_O_DISABLE404 0x40000000      /* Disable a server on a 404 response to a health-check */
 /* unused: 0x80000000 */
 
@@ -151,7 +156,8 @@ enum PR_SRV_STATE_FILE {
 #define PR_O2_RSTRICT_REQ_HDR_NAMES_DEL  0x00800000 /* remove request header names containing chars outside of [0-9a-zA-Z-] charset */
 #define PR_O2_RSTRICT_REQ_HDR_NAMES_NOOP 0x01000000 /* preserve request header names containing chars outside of [0-9a-zA-Z-] charset */
 #define PR_O2_RSTRICT_REQ_HDR_NAMES_MASK 0x01c00000 /* mask for restrict-http-header-names option */
-/* unused : 0x0000000..0x80000000 */
+
+/* unused : 0x02000000 ... 0x08000000 */
 
 /* server health checks */
 #define PR_O2_CHK_NONE  0x00000000      /* no L7 health checks configured (TCP by default) */
@@ -161,6 +167,29 @@ enum PR_SRV_STATE_FILE {
 #define PR_O2_CHK_ANY   0xF0000000      /* Mask to cover any check */
 /* end of proxy->options2 */
 
+/* bits for proxy->options3 */
+
+/*   bits for log-forward proxies */
+#define PR_O3_DONTPARSELOG       0x00000001 /* don't parse log messages */
+#define PR_O3_ASSUME_RFC6587_NTF 0x00000002 /* assume that we are going to receive just non-transparent framing messages */
+
+/* unused: 0x00000004 to 0x00000008 */
+
+#define PR_O3_LOGF_HOST_REPLACE  0x00000010
+#define PR_O3_LOGF_HOST_FILL     0x00000020
+#define PR_O3_LOGF_HOST_KEEP     0x00000040
+#define PR_O3_LOGF_HOST_APPEND   0x00000080
+#define PR_O3_LOGF_HOST          0x000000F0
+
+/* bits for hash-preserve-affinity */
+#define PR_O3_HASHAFNTY_ALWS     0x00000000 /* always preserve hash affinity */
+#define PR_O3_HASHAFNTY_MAXCONN  0x00000100 /* preserve hash affinity until maxconn is reached */
+#define PR_O3_HASHAFNTY_MAXQUEUE 0x00000200 /* preserve hash affinity until maxqueue is reached */
+#define PR_O3_HASHAFNTY_MASK     0x00000300 /* mask for hash-preserve-affinity */
+
+/* unused: 0x00000400 to  0x80000000 */
+/* end of proxy->options3 */
+
 /* Cookie settings for pr->ck_opts */
 #define PR_CK_RW        0x00000001      /* rewrite all direct cookies with the right serverid */
 #define PR_CK_IND       0x00000002      /* keep only indirect cookies */
@@ -271,6 +300,12 @@ struct error_snapshot {
 	char buf[VAR_ARRAY];                    /* copy of the beginning of the message for bufsize bytes */
 };
 
+/* Each proxy will have one occurrence of this structure per thread group */
+struct proxy_per_tgroup {
+	struct queue queue;
+	struct lbprm_per_tgrp lbprm;
+} THREAD_ALIGNED(64);
+
 struct proxy {
 	enum obj_type obj_type;                 /* object type == OBJ_TYPE_PROXY */
 	char flags;                             /* bit field PR_FL_* */
@@ -280,6 +315,7 @@ struct proxy {
 
 	int options;				/* PR_O_REDISP, PR_O_TRANSP, ... */
 	int options2;				/* PR_O2_* */
+	int options3;                           /* PR_O3_* */
 	unsigned int ck_opts;			/* PR_CK_* (cookie options) */
 	unsigned int fe_req_ana, be_req_ana;	/* bitmap of common request protocol analysers for the frontend and backend */
 	unsigned int fe_rsp_ana, be_rsp_ana;	/* bitmap of common response protocol analysers for the frontend and backend */
@@ -317,6 +353,7 @@ struct proxy {
 	int srv_act, srv_bck;			/* # of servers eligible for LB (UP|!checked) AND (enabled+weight!=0) */
 	int served;				/* # of active sessions currently being served */
 	int  cookie_len;			/* strlen(cookie_name), computed only once */
+	struct server *ready_srv;		/* a server being ready to serve requests */
 	char *cookie_domain;			/* domain used to insert the cookie */
 	char *cookie_name;			/* name of the cookie to look for */
 	char *cookie_attrs;                     /* list of attributes to add to the cookie */
@@ -356,7 +393,8 @@ struct proxy {
 	__decl_thread(HA_RWLOCK_T lock);        /* may be taken under the server's lock */
 
 	char *id, *desc;			/* proxy id (name) and description */
-	struct queue queue;			/* queued requests (pendconns) */
+	struct proxy_per_tgroup *per_tgrp;	/* array of per-tgroup stuff such as queues */
+	unsigned int queueslength;		/* Sum of the length of each queue */
 	int totpend;				/* total number of pending connections on this instance (for stats) */
 	unsigned int feconn, beconn;		/* # of active frontend and backends streams */
 	unsigned int fe_sps_lim;		/* limit on new sessions per second on the frontend */
@@ -413,6 +451,7 @@ struct proxy {
 	/* used only during configuration parsing */
 	int no_options;				/* PR_O_REDISP, PR_O_TRANSP, ... */
 	int no_options2;			/* PR_O2_* */
+	int no_options3;                        /* PR_O3_* */
 
 	struct {
 		const char *file;		/* file where the section appears */
