@@ -151,10 +151,18 @@ struct stksess {
 	int seen;                 /* 0 only when no peer has seen this entry yet */
 	struct eb32_node exp;     /* ebtree node used to hold the session in expiration tree */
 	struct eb32_node upd;     /* ebtree node used to hold the update sequence tree */
+	struct mt_list pend_updts;/* list of entries to be inserted/moved in the update sequence tree */
+	int updt_is_local;        /* is the update a local one ? */
 	struct ebmb_node key;     /* ebtree node used to hold the session in table */
 	/* WARNING! do not put anything after <keys>, it's used by the key */
 };
 
+/* stktable struct flags */
+#define STK_FL_NONE      0x0000
+#define STK_FL_RECV_ONLY 0x0001    /* table is assumed to be remotely updated only
+                                    * (never updated locally)
+                                    */
+#define STK_FL_NOPURGE   0x0002    /* if non-zero, don't purge sticky sessions when full */
 
 /* stick table */
 struct stktable {
@@ -179,11 +187,13 @@ struct stktable {
 	size_t key_size;          /* size of a key, maximum size in case of string */
 	unsigned int server_key_type; /* What type of key is used to identify servers */
 	unsigned int size;        /* maximum number of sticky sessions in table */
-	int nopurge;              /* if non-zero, don't purge sticky sessions when full */
 	int expire;               /* time to live for sticky sessions (milliseconds) */
 	int data_size;            /* the size of the data that is prepended *before* stksess */
 	int data_ofs[STKTABLE_DATA_TYPES]; /* negative offsets of present data types, or 0 if absent */
 	unsigned int data_nbelem[STKTABLE_DATA_TYPES]; /* to store nb_elem in case of array types */
+	unsigned int brates_factor; /* Factor used for IN/OUT bytes rates */
+	uint16_t flags; /* STK_FL_* flags */
+	/* 2-bytes hole */
 	union {
 		int i;
 		unsigned int u;
@@ -212,9 +222,11 @@ struct stktable {
 	THREAD_ALIGN(64);
 
 	struct eb_root updates;   /* head of sticky updates sequence tree, uses updt_lock */
+	struct mt_list *pend_updts; /* list of updates to be added to the update sequence tree, one per thread-group */
 	unsigned int update;      /* uses updt_lock */
 	unsigned int localupdate; /* uses updt_lock */
 	unsigned int commitupdate;/* used to identify the latest local updates pending for sync, uses updt_lock */
+	struct tasklet *updt_task;/* tasklet responsable for pushing the pending updates into the tree */
 
 	THREAD_ALIGN(64);
 	/* this lock is heavily used and must be on its own cache line */
@@ -223,7 +235,7 @@ struct stktable {
 	/* rarely used config stuff below (should not interfere with updt_lock) */
 	struct proxy *proxies_list; /* The list of proxies which reference this stick-table. */
 	struct {
-		const char *file;     /* The file where the stick-table is declared. */
+		const char *file;     /* The file where the stick-table is declared (global name). */
 		int line;             /* The line in this <file> the stick-table is declared. */
 	} conf;
 };
