@@ -27,6 +27,7 @@
 #define _GNU_SOURCE
 #endif
 
+#include <errno.h>
 #include <string.h>
 #include <stdio.h>
 #include <time.h>
@@ -70,7 +71,7 @@
 #define HA_ANON_PATH(key, str) hash_anon(key, str, "PATH(", ")")
 
 /* use only in a function that contains an appctx (key comes from appctx). */
-#define HA_ANON_CLI(str) hash_anon(appctx->cli_anon_key, str, "", "")
+#define HA_ANON_CLI(str) hash_anon(appctx->cli_ctx.anon_key, str, "", "")
 
 
 /*
@@ -323,6 +324,7 @@ struct sockaddr_storage *str2sa_range(const char *str, int *port, int *low, int
 /* converts <addr> and <port> into a string representation of the address and port. This is sort
  * of an inverse of str2sa_range, with some restrictions. The supported families are AF_INET,
  * AF_INET6, AF_UNIX, and AF_CUST_SOCKPAIR. If the family is unsopported NULL is returned.
+ * If port is special value '-1', then only the address is represented and <map_ports> is ignored.
  * If map_ports is true, then the sign of the port is included in the output, to indicate it is
  * relative to the incoming port. AF_INET and AF_INET6 will be in the form "<addr>:<port>".
  * AF_UNIX will either be just the path (if using a pathname) or "abns@<path>" if it is abstract.
@@ -672,7 +674,7 @@ extern const char *parse_size_ull(const char *text, ullong *ret);
 int parse_binary(const char *source, char **binstr, int *binstrlen, char **err);
 
 /* copies at most <n> characters from <src> and always terminates with '\0' */
-char *my_strndup(const char *src, int n);
+char *my_strndup(const char *src, size_t n);
 
 /*
  * search needle in haystack
@@ -1020,6 +1022,9 @@ int my_unsetenv(const char *name);
  * some expansion is made.
  */
 char *env_expand(char *in);
+int is_path_mode(mode_t mode, const char *path_fmt, ...);
+int is_file_present(const char *path_fmt, ...);
+int is_dir_present(const char *path_fmt, ...);
 uint32_t parse_line(char *in, char *out, size_t *outlen, char **args, int *nbargs, uint32_t opts, const char **errptr);
 ssize_t read_line_to_trash(const char *path_fmt, ...);
 size_t sanitize_for_printing(char *line, size_t pos, size_t width);
@@ -1028,9 +1033,27 @@ void make_word_fingerprint(uint8_t *fp, const char *word);
 int word_fingerprint_distance(const uint8_t *fp1, const uint8_t *fp2);
 
 /* debugging macro to emit messages using write() on fd #-1 so that strace sees
- * them.
+ * them. It relies on variadic macros with optional arguments so that any
+ * number of argument is accepted. If at least one argument is passed, the
+ * first one is a format string and the other ones are the arguments, exactly
+ * like printf(). The macro always prepends the function name and the location
+ * as file:line between square brackets on any line. If no format string is
+ * passed, then "\n" is used. Otherwise the caller has to deal with \n itself
+ * (format or data).
  */
-#define fddebug(msg...) do { char *_m = NULL; memprintf(&_m, ##msg); if (_m) write(-1, _m, strlen(_m)); free(_m); } while (0)
+#define fddebug(...) __fddebug(__VA_ARGS__)
+#define _fddebug(fmt, msg...) __fddebug("" ##fmt, ##msg)
+#define __fddebug(fmt, msg...) do {					\
+		char *_m = NULL;					\
+		memprintf(&_m,						\
+			  (""fmt)[0] ?					\
+			  ("[%s@%s:%d] " fmt) :				\
+			  ("[%s@%s:%d]\n"), __func__,			\
+			  __FILE__, __LINE__, ##msg);			\
+		if (_m)							\
+			write(-1, _m, strlen(_m));			\
+		free(_m);						\
+	} while (0)
 
 /* displays a <len> long memory block at <buf>, assuming first byte of <buf>
  * has address <baseaddr>. String <pfx> may be placed as a prefix in front of
@@ -1311,4 +1334,62 @@ int backup_env(void);
 int clean_env(void);
 int restore_env(void);
 
+/* helper to print the name of errno's corresponding macro (for example "EINVAL"
+ * for errno=22) instead of calling strerror(errno).
+ */
+#define CASE_ERR(err) \
+	case err: return #err
+
+static inline const char *errname(int err_num, char **out)
+{
+	/* only currently used errno values, please, add a new one, if you
+	 * start using it in the code.
+	 */
+	switch (err_num) {
+	case 0: return "SUCCESS";
+	CASE_ERR(EPERM);
+	CASE_ERR(ENOENT);
+	CASE_ERR(EINTR);
+	CASE_ERR(EIO);
+	CASE_ERR(E2BIG);
+	CASE_ERR(EBADF);
+	CASE_ERR(ECHILD);
+	CASE_ERR(EAGAIN);
+	CASE_ERR(ENOMEM);
+	CASE_ERR(EACCES);
+	CASE_ERR(EFAULT);
+	CASE_ERR(EINVAL);
+	CASE_ERR(ENFILE);
+	CASE_ERR(EMFILE);
+	CASE_ERR(ENOSPC);
+	CASE_ERR(ERANGE);
+	CASE_ERR(ENOSYS);
+	CASE_ERR(EADDRINUSE);
+	CASE_ERR(ECONNABORTED);
+	CASE_ERR(ECONNRESET);
+	CASE_ERR(EINPROGRESS);
+	CASE_ERR(ENOTCONN);
+	CASE_ERR(EADDRNOTAVAIL);
+	CASE_ERR(ECONNREFUSED);
+	CASE_ERR(ENETUNREACH);
+	CASE_ERR(EPROTO);
+	CASE_ERR(ENOTSOCK);
+	CASE_ERR(EMSGSIZE);
+	CASE_ERR(EPROTONOSUPPORT);
+	CASE_ERR(EAFNOSUPPORT);
+	CASE_ERR(ENOBUFS);
+	CASE_ERR(EISCONN);
+	CASE_ERR(ETIMEDOUT);
+	CASE_ERR(EALREADY);
+
+	default:
+		memprintf(out, "%d", err_num);
+		return *out;
+	}
+}
+
+int path_base(const char *path, const char *base, char *dst, char **err);
+
+void ha_freearray(char ***array);
+
 #endif /* _HAPROXY_TOOLS_H */
