@@ -545,7 +545,6 @@ error:
 /* Initialize stuff once the section is parsed */
 static int cfg_postsection_acme()
 {
-	struct acme_cfg *cur_acme = acme_cfgs;
 	struct ckch_store *store;
 	EVP_PKEY *key = NULL;
 	BIO *bio = NULL;
@@ -770,7 +769,6 @@ static void acme_ctx_destroy(struct acme_ctx *ctx)
 
 	X509_REQ_free(ctx->req);
 
-	MT_LIST_DELETE(&ctx->el);
 
 	free(ctx);
 }
@@ -1225,7 +1223,7 @@ int acme_req_finalize(struct task *task, struct acme_ctx *ctx, char **errmsg)
 	csr->data = ret;
 
 	chunk_printf(req_in, "{ \"csr\": \"%.*s\" }", (int)csr->data, csr->area);
-	free(data);
+	OPENSSL_free(data);
 
 
 	if (acme_jws_payload(req_in, ctx->nonce, ctx->finalize, ctx->cfg->account.pkey, ctx->kid, req_out, errmsg) != 0)
@@ -2276,8 +2274,10 @@ abort:
 	ha_free(&errmsg);
 
 end:
-	MT_LIST_UNLOCK_FULL(&ctx->el, tmp);
 	acme_del_acme_ctx_map(ctx);
+	/* unlink ctx from the mtlist then destroy */
+	mt_list_unlock_link(tmp);
+	mt_list_unlock_self(&ctx->el);
 	acme_ctx_destroy(ctx);
 	task_destroy(task);
 	task = NULL;
@@ -2299,7 +2299,7 @@ wait:
  */
 int acme_will_expire(struct ckch_store *store)
 {
-	int diff = 0;
+	time_t diff = 0;
 	time_t notAfter = 0;
 	time_t notBefore = 0;
 
@@ -2310,13 +2310,14 @@ int acme_will_expire(struct ckch_store *store)
 	notAfter = x509_get_notafter_time_t(store->data->cert);
 	notBefore = x509_get_notbefore_time_t(store->data->cert);
 
-	if (notAfter >= 0 && notBefore >= 0) {
+	if ((notAfter >= 0 && notBefore >= 0)
+	   && (notAfter > notBefore)) {
 		diff = (notAfter - notBefore) / 12; /* validity period / 12 */
 	} else {
 		diff = 7 * 24 * 60 * 60; /* default to 7 days */
 	}
 
-	if (date.tv_sec + diff > notAfter)
+	if (notAfter - diff <= date.tv_sec)
 		return 1;
 
 	return 0;
@@ -2328,7 +2329,7 @@ int acme_will_expire(struct ckch_store *store)
  */
 time_t acme_schedule_date(struct ckch_store *store)
 {
-	int diff = 0;
+	time_t diff = 0;
 	time_t notAfter = 0;
 	time_t notBefore = 0;
 
@@ -2342,7 +2343,8 @@ time_t acme_schedule_date(struct ckch_store *store)
 	notAfter = x509_get_notafter_time_t(store->data->cert);
 	notBefore = x509_get_notbefore_time_t(store->data->cert);
 
-	if (notAfter >= 0 && notBefore >= 0) {
+	if ((notAfter >= 0 && notBefore >= 0)
+	   && (notAfter > notBefore)) {
 		diff = (notAfter - notBefore) / 12; /* validity period / 12 */
 	} else {
 		diff = 7 * 24 * 60 * 60; /* default to 7 days */
@@ -2742,7 +2744,7 @@ static int cli_acme_ps(char **args, char *payload, struct appctx *appctx, void *
 static struct cli_kw_list cli_kws = {{ },{
 	{ { "acme", "renew", NULL },           "acme renew <certfile>                   : renew a certificate using the ACME protocol", cli_acme_renew_parse, NULL, NULL, NULL, 0 },
 	{ { "acme", "status", NULL },          "acme status                             : show status of certificates configured with ACME", cli_acme_ps, cli_acme_status_io_handler, NULL, NULL, 0 },
-	{ { "acme", "challenge_ready", NULL }, "acme challenge_ready <certfile> domain <domain> : show status of certificates configured with ACME", cli_acme_chall_ready_parse, NULL, NULL, NULL, 0 },
+	{ { "acme", "challenge_ready", NULL }, "acme challenge_ready <certfile> domain <domain> : notify HAProxy that the ACME challenge is ready", cli_acme_chall_ready_parse, NULL, NULL, NULL, 0 },
 	{ { NULL }, NULL, NULL, NULL }
 }};
 
