@@ -174,35 +174,49 @@ int act_resolution_error_cb(struct resolv_requester *requester, int error_code)
 }
 
 /* Parse a set-timeout rule statement. It first checks if the timeout name is
- * valid and returns it in <name>. Then the timeout is parsed as a plain value
- * and * returned in <out_timeout>. If there is a parsing error, the value is
- * reparsed as an expression and returned in <expr>.
+ * valid and proxy is capable of handling it, and returns it in <rule->arg.timeout.type>.
+ * Then the timeout is parsed as a plain value and * returned in <rule->arg.timeout.value>.
+ * If there is a parsing error, the value is reparsed as an expression and
+ * returned in <rule->arg.timeout.expr>.
  *
  * Returns -1 if the name is invalid or neither a time or an expression can be
  * parsed, or if the timeout value is 0.
  */
-int cfg_parse_rule_set_timeout(const char **args, int idx, int *out_timeout,
-                               enum act_timeout_name *name,
-                               struct sample_expr **expr, char **err,
-                               const char *file, int line, struct arg_list *al)
+int cfg_parse_rule_set_timeout(const char **args, int idx, struct act_rule *rule,
+			       struct proxy *px, char **err)
 {
 	const char *res;
 	const char *timeout_name = args[idx++];
 
 	if (strcmp(timeout_name, "server") == 0) {
-		*name = ACT_TIMEOUT_SERVER;
+		if (!(px->cap & PR_CAP_BE)) {
+			memprintf(err, "'%s' has no backend capability", px->id);
+			return -1;
+		}
+		rule->arg.timeout.type = ACT_TIMEOUT_SERVER;
 	}
 	else if (strcmp(timeout_name, "tunnel") == 0) {
-		*name = ACT_TIMEOUT_TUNNEL;
+		if (!(px->cap & PR_CAP_BE)) {
+			memprintf(err, "'%s' has no backend capability", px->id);
+			return -1;
+		}
+		rule->arg.timeout.type = ACT_TIMEOUT_TUNNEL;
+	}
+	else if (strcmp(timeout_name, "client") == 0) {
+		if (!(px->cap & PR_CAP_FE)) {
+			memprintf(err, "'%s' has no frontend capability", px->id);
+			return -1;
+		}
+		rule->arg.timeout.type = ACT_TIMEOUT_CLIENT;
 	}
 	else {
 		memprintf(err,
-		          "'set-timeout' rule supports 'server'/'tunnel' (got '%s')",
+		          "'set-timeout' rule supports 'server'/'tunnel'/'client' (got '%s')",
 		          timeout_name);
 		return -1;
 	}
 
-	res = parse_time_err(args[idx], (unsigned int *)out_timeout, TIME_UNIT_MS);
+	res = parse_time_err(args[idx], (unsigned int *)&rule->arg.timeout.value, TIME_UNIT_MS);
 	if (res == PARSE_TIME_OVER) {
 		memprintf(err, "timer overflow in argument '%s' to rule 'set-timeout %s' (maximum value is 2147483647 ms or ~24.8 days)",
 			  args[idx], timeout_name);
@@ -215,14 +229,15 @@ int cfg_parse_rule_set_timeout(const char **args, int idx, int *out_timeout,
 	}
 	/* res not NULL, parsing error */
 	else if (res) {
-		*expr = sample_parse_expr((char **)args, &idx, file, line, err, al, NULL);
-		if (!*expr) {
+		rule->arg.timeout.expr = sample_parse_expr((char **)args, &idx, px->conf.args.file,
+							   px->conf.args.line, err, &px->conf.args, NULL);
+		if (!rule->arg.timeout.expr) {
 			memprintf(err, "unexpected character '%c' in rule 'set-timeout %s'", *res, timeout_name);
 			return -1;
 		}
 	}
 	/* res NULL, parsing ok but value is 0 */
-	else if (!(*out_timeout)) {
+	else if (!(rule->arg.timeout.value)) {
 		memprintf(err, "null value is not valid for a 'set-timeout %s' rule",
 			  timeout_name);
 		return -1;
