@@ -115,7 +115,7 @@ void chunk_frm_appendf(struct buffer *buf, const struct quic_frame *frm)
 	{
 		const struct qf_crypto *crypto_frm = &frm->crypto;
 		chunk_appendf(buf, " cfoff=%llu cflen=%llu",
-		              (ull)crypto_frm->offset, (ull)crypto_frm->len);
+		              (ull)crypto_frm->offset_node.key, (ull)crypto_frm->len);
 		break;
 	}
 	case QUIC_FT_RESET_STREAM:
@@ -430,12 +430,12 @@ static int quic_build_crypto_frame(unsigned char **pos, const unsigned char *end
 	const struct quic_enc_level *qel = crypto_frm->qel;
 	size_t offset, len;
 
-	if (!quic_enc_int(pos, end, crypto_frm->offset) ||
+	if (!quic_enc_int(pos, end, crypto_frm->offset_node.key) ||
 	    !quic_enc_int(pos, end, crypto_frm->len) || end - *pos < crypto_frm->len)
 		return 0;
 
 	len = crypto_frm->len;
-	offset = crypto_frm->offset;
+	offset = crypto_frm->offset_node.key;
 	while (len) {
 		int idx;
 		size_t to_copy;
@@ -463,7 +463,7 @@ static int quic_parse_crypto_frame(struct quic_frame *frm, struct quic_conn *qc,
 {
 	struct qf_crypto *crypto_frm = &frm->crypto;
 
-	if (!quic_dec_int(&crypto_frm->offset, pos, end) ||
+	if (!quic_dec_int((uint64_t *)&crypto_frm->offset_node.key, pos, end) ||
 	    !quic_dec_int(&crypto_frm->len, pos, end) || end - *pos < crypto_frm->len)
 		return 0;
 
@@ -1355,7 +1355,7 @@ size_t quic_strm_frm_fillbuf(size_t room, struct quic_frame *frm, size_t *split)
 	}
 	else if (frm->type == QUIC_FT_CRYPTO) {
 		total = 1;
-		total += quic_int_getsize(frm->crypto.offset);
+		total += quic_int_getsize(frm->crypto.offset_node.key);
 		payload = frm->crypto.len;
 	}
 	else {
@@ -1430,12 +1430,12 @@ struct quic_frame *quic_strm_frm_split(struct quic_frame *frm, uint64_t split)
 	}
 	else if (frm->type == QUIC_FT_CRYPTO) {
 		new->crypto.qel = frm->crypto.qel;
-		new->crypto.offset = frm->crypto.offset;
+		new->crypto.offset_node.key = frm->crypto.offset_node.key;
 		new->crypto.len = split;
 
 		/* Advance original frame to point to the remaining data. */
 		frm->crypto.len -= split;
-		frm->crypto.offset += split;
+		frm->crypto.offset_node.key += split;
 	}
 	else {
 		/* Function must only be used with STREAM or CRYPTO frames. */
