@@ -359,7 +359,7 @@ static int cli_parse_show_events(char **args, char *payload, struct appctx *appc
 
 	if (!*args[1]) {
 		/* no arg => report the list of supported sink */
-		chunk_printf(&trash, "Supported events sinks are listed below. Add -w(wait), -n(new). Any key to stop\n");
+		chunk_printf(&trash, "Supported events sinks are listed below. Add -0(zero), -w(wait), -n(new). Any key to stop.\n");
 		list_for_each_entry(sink, &sink_list, sink_list) {
 			chunk_appendf(&trash, "    %-10s : type=%s, %u dropped, %s\n",
 				      sink->name,
@@ -389,6 +389,8 @@ static int cli_parse_show_events(char **args, char *payload, struct appctx *appc
 			ring_flags |= RING_WF_WAIT_MODE;
 		else if (strcmp(args[arg], "-n") == 0)
 			ring_flags |= RING_WF_SEEK_NEW;
+		else if (strcmp(args[arg], "-0") == 0)
+			ring_flags |= RING_WF_END_ZERO;
 		else if (strcmp(args[arg], "-nw") == 0 || strcmp(args[arg], "-wn") == 0)
 			ring_flags |= RING_WF_WAIT_MODE | RING_WF_SEEK_NEW;
 		else
@@ -400,10 +402,12 @@ static int cli_parse_show_events(char **args, char *payload, struct appctx *appc
 /* Pre-configures a ring proxy to emit connections */
 void sink_setup_proxy(struct proxy *px)
 {
-	px->be_counters.last_change = ns_to_sec(now_ns);
-	px->cap = PR_CAP_BE;
 	px->maxconn = 0;
-	px->conn_retries = 1;
+	px->conn_retries = 1; /* FIXME ignored since 91e785ed
+	                       * ("MINOR: stream: Rely on a per-stream max connection retries value")
+	                       * If this is really expected this should be set on the stream directly
+	                       * because the proxy lacks the CAP_FE so this setting is not considered
+	                       */
 	px->timeout.server = TICK_ETERNITY;
 	px->timeout.client = TICK_ETERNITY;
 	px->timeout.connect = TICK_ETERNITY;
@@ -414,7 +418,7 @@ void sink_setup_proxy(struct proxy *px)
 }
 
 static void _sink_forward_io_handler(struct appctx *appctx,
-                                     ssize_t (*msg_handler)(void *ctx, struct ist v1, struct ist v2, size_t ofs, size_t len))
+                                     ssize_t (*msg_handler)(void *ctx, struct ist v1, struct ist v2, size_t ofs, size_t len, char delim))
 {
 	struct stconn *sc = appctx_sc(appctx);
 	struct sink_forward_target *sft = appctx->svcctx;
@@ -448,7 +452,7 @@ static void _sink_forward_io_handler(struct appctx *appctx,
 	MT_LIST_DELETE(&appctx->wait_entry);
 
 	ret = ring_dispatch_messages(ring, appctx, &sft->ofs, &last_ofs, 0,
-	                             msg_handler, &processed);
+	                             msg_handler, '\n', &processed);
 	sft->e_processed += processed;
 
 	/* if server's max-reuse is set (>= 0), destroy the applet once the
@@ -678,7 +682,7 @@ static struct task *process_sink_forward(struct task * task, void *context, unsi
 		while (sft) {
 			HA_SPIN_LOCK(SFT_LOCK, &sft->lock);
 			/* If appctx is NULL, start a new session and perform the appctx
-			 * assigment right away since the applet is not supposed to change
+			 * assignment right away since the applet is not supposed to change
 			 * during the session lifetime. By doing the assignment now we
 			 * make sure to start the session exactly once.
 			 *
@@ -686,7 +690,7 @@ static struct task *process_sink_forward(struct task * task, void *context, unsi
 			 * establishment attempt per second.
 			 */
 			if (!sft->appctx) {
-				int tempo = tick_add(sft->last_conn, MS_TO_TICKS(1000));
+				uint tempo = sft->last_conn + MS_TO_TICKS(1000);
 
 				if (sft->last_conn == TICK_ETERNITY || tick_is_expired(tempo, now_ms))
 					sft->appctx = sink_forward_session_create(sink, sft);
@@ -828,15 +832,11 @@ static struct sink *sink_new_ringbuf(const char *id, const char *description,
 	struct proxy *p = NULL; // forward_px
 
 	/* allocate new proxy to handle forwards */
-	p = calloc(1, sizeof(*p));
-	if (!p) {
-		memprintf(err_msg, "out of memory");
+	p = alloc_new_proxy(id, PR_CAP_BE, err_msg);
+	if (!p)
 		goto err;
-	}
 
-	init_new_proxy(p);
 	sink_setup_proxy(p);
-	p->id = strdup(id);
 	p->conf.args.file = p->conf.file = copy_file_name(file);
 	p->conf.args.line = p->conf.line = linenum;
 
@@ -1327,7 +1327,7 @@ struct sink *sink_new_from_srv(struct server *srv, const char *from)
 	/* directly create a sink of BUF type, and use UNSPEC log format to
 	 * inherit from caller fmt in sink_write()
 	 *
-	 * sink_name must be unique to prevent existing sink from being re-used
+	 * sink_name must be unique to prevent existing sink from being reused
 	 */
 	sink = sink_new_buf(sink_name, trash.area, LOG_FORMAT_UNSPEC, bufsize);
 	ha_free(&sink_name); // no longer needed
@@ -1430,6 +1430,7 @@ static void sink_init()
 	sink_new_fd("stdout", "standard output (fd#1)", LOG_FORMAT_RAW, 1);
 	sink_new_fd("stderr", "standard output (fd#2)", LOG_FORMAT_RAW, 2);
 	sink_new_buf("buf0",  "in-memory ring buffer", LOG_FORMAT_TIMED, 1048576);
+	sink_new_buf("dpapi",  "DPAPI ring buffer", LOG_FORMAT_TIMED, 1048576);
 }
 
 static int sink_postcheck()
@@ -1462,7 +1463,7 @@ REGISTER_POST_CHECK(sink_postcheck);
 REGISTER_POST_DEINIT(sink_deinit);
 
 static struct cli_kw_list cli_kws = {{ },{
-	{ { "show", "events", NULL }, "show events [<sink>] [-w] [-n]          : show event sink state", cli_parse_show_events, NULL, NULL },
+	{ { "show", "events", NULL }, "show events [<sink>] [-w] [-n] [-0]     : show event sink state", cli_parse_show_events, NULL, NULL },
 	{{},}
 }};
 
