@@ -88,9 +88,13 @@ struct connection *sock_accept_conn(struct listener *l, int *status)
 	 * the legacy accept() + fcntl().
 	 */
 	if (unlikely(accept4_broken) ||
+	    /* Albeit it appears it does not make sense to carry on with accept
+	     * if we encounter EPERM, some old embedded ARM Linux 2.6.x sets as
+	     * such instead of ENOSYS.
+	     */
 	    (((cfd = accept4(l->rx.fd, (struct sockaddr*)addr, &laddr,
 	                     SOCK_NONBLOCK | (master ? SOCK_CLOEXEC : 0))) == -1) &&
-	     (errno == ENOSYS || errno == EINVAL || errno == EBADF) &&
+	     (errno == ENOSYS || errno == EINVAL || errno == EBADF || errno == EPERM) &&
 	     ((accept4_broken = 1))))
 #endif
 	{
@@ -376,7 +380,7 @@ void sock_unbind(struct receiver *rx)
 		return;
 
 	if (!stopping && master &&
-	    rx->flags & RX_F_INHERITED_FD)
+	    rx->flags & RX_F_INHERITED)
 		return;
 
 	rx->flags &= ~RX_F_BOUND;
@@ -482,7 +486,7 @@ int sock_get_old_sockets(const char *unixsocket)
 
 		/* dst_fd is always open in the worker process context because
 		 * it's inherited from the master via -x cmd option. It's closed
-		 * futher in main (after bind_listeners()) and not here for the
+		 * further in main (after bind_listeners()) and not here for the
 		 * simplicity. In main(), after bind_listeners(), it's safe just
 		 * to loop over all workers list, launched before this reload and
 		 * to close its ipc_fd[0], thus we also close this fd. If we
@@ -957,7 +961,7 @@ int sock_conn_check(struct connection *conn)
 		 *
 		 * We have no explanation for now. Why epoll report the
 		 * connection is closed while the connect() it able to validate
-		 * it ? no idea. But, it seems reasonnable in this case, and if
+		 * it ? no idea. But, it seems reasonable in this case, and if
 		 * no error was reported, to remove the the HUP flag. At worst, if
 		 * the connection is really closed, this will be reported later.
 		 *
@@ -988,8 +992,10 @@ int sock_conn_check(struct connection *conn)
 		if (errno == EALREADY || errno == EINPROGRESS)
 			goto wait;
 
-		if (errno && errno != EISCONN)
+		if (errno && errno != EISCONN) {
+			conn_report_term_evt(conn, tevt_loc_fd, fd_tevt_type_connect_err);
 			goto out_error;
+		}
 	}
 
  done:
@@ -1017,8 +1023,10 @@ int sock_conn_check(struct connection *conn)
 	 * in some corner cases while the system disagrees and reports an error
 	 * on the FD.
 	 */
-	if (fdtab[fd].state & FD_POLL_ERR)
+	if (fdtab[fd].state & FD_POLL_ERR) {
+		conn_report_term_evt(conn, tevt_loc_fd, fd_tevt_type_connect_poll_err);
 		goto out_error;
+	}
 
 	fd_cant_send(fd);
 	fd_want_send(fd);
