@@ -700,3 +700,241 @@ nosec:
 
 	return -1;
 }
+
+/* Return the nofAfter value as as string extracted from an X509 certificate
+ * The returned buffer is static and thread local.
+ */
+const char *x509_get_notafter(X509 *cert)
+{
+	BIO *bio = NULL;
+	int write;
+	static THREAD_LOCAL char buf[256];
+
+	memset(buf, 0, sizeof(buf));
+
+	if ((bio = BIO_new(BIO_s_mem())) ==  NULL)
+		goto end;
+	if (ASN1_TIME_print(bio, X509_getm_notAfter(cert)) == 0)
+		goto end;
+	write = BIO_read(bio, buf, sizeof(buf)-1);
+	buf[write] = '\0';
+	BIO_free(bio);
+
+	return buf;
+
+end:
+	BIO_free(bio);
+	return NULL;
+}
+
+/* Return the nofBefore value as as string extracted from an X509 certificate
+ * The returned buffer is static and thread local.
+ */
+const char *x509_get_notbefore(X509 *cert)
+{
+	BIO *bio = NULL;
+	int write;
+	static THREAD_LOCAL char buf[256];
+
+	memset(buf, 0, sizeof(buf));
+
+	if ((bio = BIO_new(BIO_s_mem())) ==  NULL)
+		goto end;
+	if (ASN1_TIME_print(bio, X509_getm_notBefore(cert)) == 0)
+		goto end;
+	write = BIO_read(bio, buf, sizeof(buf)-1);
+	buf[write] = '\0';
+	BIO_free(bio);
+
+	return buf;
+
+end:
+	BIO_free(bio);
+	return NULL;
+}
+
+#ifdef HAVE_ASN1_TIME_TO_TM
+/* Takes a ASN1_TIME and converts it into a time_t */
+time_t ASN1_to_time_t(ASN1_TIME *asn1_time)
+{
+	struct tm tm;
+	time_t ret = -1;
+
+	if (ASN1_TIME_to_tm(asn1_time, &tm) == 0)
+		goto error;
+
+	ret  = my_timegm(&tm);
+error:
+	return ret;
+}
+
+/* return the notAfter date of a X509 certificate in a time_t format */
+time_t x509_get_notafter_time_t(X509 *cert)
+{
+	time_t ret = -1;
+	ASN1_TIME *asn1_time;
+
+	if ((asn1_time = X509_getm_notAfter(cert)) == NULL)
+		goto error;
+
+	ret = ASN1_to_time_t(asn1_time);
+
+error:
+	return ret;
+}
+
+/* return the notBefore date of a X509 certificate in a time_t format */
+time_t x509_get_notbefore_time_t(X509 *cert)
+{
+	time_t ret = -1;
+	ASN1_TIME *asn1_time;
+
+	if ((asn1_time = X509_getm_notBefore(cert)) == NULL)
+		goto error;
+
+	ret = ASN1_to_time_t(asn1_time);
+
+error:
+	return ret;
+}
+#endif
+
+/* https://datatracker.ietf.org/doc/html/rfc8422#appendix-A */
+/* SECG to NIST curves name */
+static struct curves { char *name; int nid; } curves_list [] =
+{
+	{ "secp256r1",  NID_X9_62_prime256v1 },
+	{ "prime256v1", NID_X9_62_prime256v1 },
+	{ "P-256",      NID_X9_62_prime256v1 },
+
+	{ "secp384r1",  NID_secp384r1 },
+	{ "P-384",      NID_secp384r1 },
+
+	{ "secp521r1",  NID_secp521r1 },
+	{ "P-521",      NID_secp521r1 },
+	{ NULL,         0 },
+};
+
+/* convert a curves name to a openssl NID */
+int curves2nid(const char *curve)
+{
+	struct curves *curves = curves_list;
+
+	while (curves->name) {
+		if (strcmp(curve, curves->name) == 0)
+			return curves->nid;
+		curves++;
+	}
+	return -1;
+}
+
+/* convert an OpenSSL NID to a NIST curves name */
+const char *nid2nist(int nid)
+{
+	switch (nid) {
+		case NID_X9_62_prime256v1: return "P-256";
+		case NID_secp384r1:        return "P-384";
+		case NID_secp521r1:        return "P-521";
+		default:                   return NULL;
+	}
+}
+
+
+/* https://datatracker.ietf.org/doc/html/rfc8446#section-4.2.3
+ * https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-signaturescheme
+ * Sigalg identifier to sigalg name table.
+ * Some TLSv1.2 combinations are included as well to ease debugging. */
+static struct sigalgs { const char *name; int sigalg; } sigalgs_list [] =
+{
+	/* RSASSA-PKCS1-v1_5 algorithms */
+	{ "rsa_pkcs1_sha256", 0x0401 },
+	{ "rsa_pkcs1_sha384", 0x0501 },
+	{ "rsa_pkcs1_sha512", 0x0601 },
+
+	/* ECDSA algorithms */
+	{ "ecdsa_secp256r1_sha256", 0x0403 },
+	{ "ecdsa_secp384r1_sha384", 0x0503 },
+	{ "ecdsa_secp521r1_sha512", 0x0603 },
+
+	/* RSASSA-PSS algorithms with public key OID rsaEncryption */
+	{ "rsa_pss_rsae_sha256", 0x0804 },
+	{ "rsa_pss_rsae_sha384", 0x0805 },
+	{ "rsa_pss_rsae_sha512", 0x0806 },
+
+	/* EdDSA algorithms */
+	{ "ed25519", 0x0807 },
+	{ "ed448", 0x0808 },
+
+	/* RSASSA-PSS algorithms with public key OID RSASSA-PSS */
+	{ "rsa_pss_pss_sha256", 0x0809 },
+	{ "rsa_pss_pss_sha384", 0x080a },
+	{ "rsa_pss_pss_sha512", 0x080b },
+
+	/* Legacy algorithms */
+	{ "rsa_pkcs1_sha1", 0x0201 },
+	{ "ecdsa_sha1", 0x0203 },
+
+
+	/* Other IANA codes */
+	/* https://datatracker.ietf.org/doc/draft-davidben-tls13-pkcs1/00/ */
+	{ "rsa_pkcs1_sha256_legacy", 0x0420 },
+	{ "rsa_pkcs1_sha384_legacy", 0x0520 },
+	{ "rsa_pkcs1_sha512_legacy", 0x0620 },
+
+	/* https://datatracker.ietf.org/doc/draft-wang-tls-raw-public-key-with-ibc/02/ */
+	{ "eccsi_sha256", 0x0704 },
+	{ "iso_ibs1", 0x0705 },
+	{ "iso_ibs2", 0x0706 },
+	{ "iso_chinese_ibs", 0x0707 },
+
+	/* RFC 8998 */
+	{ "sm2sig_sm3", 0x0708 },
+
+	/* RFC 9367 */
+	{ "gostr34102012_256a", 0x0709 },
+	{ "gostr34102012_256b", 0x070A },
+	{ "gostr34102012_256c", 0x070B },
+	{ "gostr34102012_256d", 0x070C },
+	{ "gostr34102012_512a", 0x070D },
+	{ "gostr34102012_512b", 0x070E },
+	{ "gostr34102012_512c", 0x070F },
+
+	/* RFC 8734 */
+	{ "ecdsa_brainpoolP256r1tls13_sha256", 0x081A },
+	{ "ecdsa_brainpoolP384r1tls13_sha384", 0x081B },
+	{ "ecdsa_brainpoolP512r1tls13_sha512", 0x081C },
+
+
+	/* TLSv1.2 backward compatibility */
+	{ "dsa_sha256", 0x0402 },
+	{ "dsa_sha384", 0x0502 },
+	{ "dsa_sha512", 0x0602 },
+	{ "dsa_sha224", 0x0302 },
+	{ "dsa_sha1", 0x0202 },
+
+	{ "ecdsa_sha224", 0x0303 },
+	{ "ecdsa_sha1", 0x0203 },
+
+
+	/* RFC 9189 */
+	{ "gostr34102012_256_intrinsic", 0x0840 },
+	{ "gostr34102012_512_intrinsic", 0x0841 },
+
+	{ NULL, 0 }
+};
+
+/* Convert a signature algorithm identifier (2 bytes) to name */
+const char *sigalg2str(int sigalg)
+{
+	struct sigalgs *item = sigalgs_list;
+
+	while (item->name) {
+		if (item->sigalg == sigalg)
+			return item->name;
+
+		++item;
+	}
+
+	return NULL;
+}
+
